@@ -3,10 +3,19 @@ import datetime
 import decimal
 import inspect
 import logging
+import re
 import six
 import sys
+import uuid
 import weakref
 
+try:
+    import ipaddress
+except ImportError:
+    import ipaddr as ipaddress
+
+from wsme import exc
+
 log = logging.getLogger(__name__)
 
 #: The 'str' (python 2) or 'bytes' (python 3) type.
@@ -31,7 +40,11 @@ class ArrayType(object):
         return hash(self.item_type)
 
     def __eq__(self, other):
-        return self.item_type == other.item_type
+        return isinstance(other, ArrayType) \
+            and self.item_type == other.item_type
+
+    def sample(self):
+        return [getattr(self.item_type, 'sample', self.item_type)()]
 
     @property
     def item_type(self):
@@ -45,7 +58,7 @@ class ArrayType(object):
             return
         if not isinstance(value, list):
             raise ValueError("Wrong type. Expected '[%s]', got '%s'" % (
-                    self.item_type, type(value)
+                self.item_type, type(value)
             ))
         return [
             validate_value(self.item_type, item)
@@ -56,7 +69,7 @@ class ArrayType(object):
 class DictType(object):
     def __init__(self, key_type, value_type):
         if key_type not in pod_types:
-            raise ValueError("Dictionnaries key can only be a pod type")
+            raise ValueError("Dictionaries key can only be a pod type")
         self.key_type = key_type
         if iscomplex(value_type):
             self._value_type = weakref.ref(value_type)
@@ -66,6 +79,11 @@ class DictType(object):
     def __hash__(self):
         return hash((self.key_type, self.value_type))
 
+    def sample(self):
+        key = getattr(self.key_type, 'sample', self.key_type)()
+        value = getattr(self.value_type, 'sample', self.value_type)()
+        return {key: value}
+
     @property
     def value_type(self):
         if isinstance(self._value_type, weakref.ref):
@@ -76,8 +94,8 @@ class DictType(object):
     def validate(self, value):
         if not isinstance(value, dict):
             raise ValueError("Wrong type. Expected '{%s: %s}', got '%s'" % (
-                    self.key_type, self.value_type, type(value)
-                ))
+                self.key_type, self.value_type, type(value)
+            ))
         return dict((
             (
                 validate_value(self.key_type, key),
@@ -125,6 +143,139 @@ class BinaryType(UserType):
 binary = BinaryType()
 
 
+class IntegerType(UserType):
+    """
+    A simple integer type. Can validate a value range.
+
+    :param minimum: Possible minimum value
+    :param maximum: Possible maximum value
+
+    Example::
+
+        Price = IntegerType(minimum=1)
+
+    """
+    basetype = int
+    name = "integer"
+
+    def __init__(self, minimum=None, maximum=None):
+        self.minimum = minimum
+        self.maximum = maximum
+
+    @staticmethod
+    def frombasetype(value):
+        return int(value) if value is not None else None
+
+    def validate(self, value):
+        if self.minimum is not None and value < self.minimum:
+            error = 'Value should be greater or equal to %s' % self.minimum
+            raise ValueError(error)
+
+        if self.maximum is not None and value > self.maximum:
+            error = 'Value should be lower or equal to %s' % self.maximum
+            raise ValueError(error)
+
+        return value
+
+
+class StringType(UserType):
+    """
+    A simple string type. Can validate a length and a pattern.
+
+    :param min_length: Possible minimum length
+    :param max_length: Possible maximum length
+    :param pattern: Possible string pattern
+
+    Example::
+
+        Name = StringType(min_length=1, pattern='^[a-zA-Z ]*$')
+
+    """
+    basetype = six.string_types
+    name = "string"
+
+    def __init__(self, min_length=None, max_length=None, pattern=None):
+        self.min_length = min_length
+        self.max_length = max_length
+        if isinstance(pattern, six.string_types):
+            self.pattern = re.compile(pattern)
+        else:
+            self.pattern = pattern
+
+    def validate(self, value):
+        if not isinstance(value, self.basetype):
+            error = 'Value should be string'
+            raise ValueError(error)
+
+        if self.min_length is not None and len(value) < self.min_length:
+            error = 'Value should have a minimum character requirement of %s' \
+                    % self.min_length
+            raise ValueError(error)
+
+        if self.max_length is not None and len(value) > self.max_length:
+            error = 'Value should have a maximum character requirement of %s' \
+                    % self.max_length
+            raise ValueError(error)
+
+        if self.pattern is not None and not self.pattern.search(value):
+            error = 'Value should match the pattern %s' % self.pattern
+            raise ValueError(error)
+
+        return value
+
+
+class IPv4AddressType(UserType):
+    """
+    A simple IPv4 type.
+    """
+    basetype = six.string_types
+    name = "ipv4address"
+
+    @staticmethod
+    def validate(value):
+        try:
+            ipaddress.IPv4Address(value)
+        except ipaddress.AddressValueError:
+            error = 'Value should be IPv4 format'
+            raise ValueError(error)
+
+
+class IPv6AddressType(UserType):
+    """
+    A simple IPv6 type.
+    """
+    basetype = six.string_types
+    name = "ipv6address"
+
+    @staticmethod
+    def validate(value):
+        try:
+            ipaddress.IPv6Address(value)
+        except ipaddress.AddressValueError:
+            error = 'Value should be IPv6 format'
+            raise ValueError(error)
+
+
+class UuidType(UserType):
+    """
+    A simple UUID type.
+
+    This type allows not only UUID having dashes but also UUID not
+    having dashes. For example, '6a0a707c-45ef-4758-b533-e55adddba8ce'
+    and '6a0a707c45ef4758b533e55adddba8ce' are distinguished as valid.
+    """
+    basetype = six.string_types
+    name = "uuid"
+
+    @staticmethod
+    def validate(value):
+        try:
+            return six.text_type((uuid.UUID(value)))
+        except (TypeError, ValueError, AttributeError):
+            error = 'Value should be UUID format'
+            raise ValueError(error)
+
+
 class Enum(UserType):
     """
     A simple enumeration type. Can be based on any non-complex type.
@@ -145,13 +296,13 @@ class Enum(UserType):
         self.values = set(values)
         name = kw.pop('name', None)
         if name is None:
-            name = "Enum(%s)" % ', '.join((str(v) for v in values))
+            name = "Enum(%s)" % ', '.join((six.text_type(v) for v in values))
         self.name = name
 
     def validate(self, value):
         if value not in self.values:
-            raise ValueError("Value '%s' is invalid (should be one of: %s)" % (
-                value, ', '.join(self.values)))
+            raise ValueError("Value should be one of: %s" %
+                             ', '.join(map(six.text_type, self.values)))
         return value
 
     def tobasetype(self, value):
@@ -169,19 +320,29 @@ class UnsetType(object):
         def __bool__(self):
             return False
 
+    def __repr__(self):
+        return 'Unset'
+
 Unset = UnsetType()
 
+#: A special type that corresponds to the host framework request object.
+#: It can only be used in the function parameters, and if so the request object
+#: of the host framework will be passed to the function.
+HostRequest = object()
+
 
 pod_types = six.integer_types + (
     bytes, text, float, bool)
 dt_types = (datetime.date, datetime.time, datetime.datetime)
 extra_types = (binary, decimal.Decimal)
 native_types = pod_types + dt_types + extra_types
+# The types for which we allow promotion to certain numbers.
+_promotable_types = six.integer_types + (text, bytes)
 
 
 def iscomplex(datatype):
     return inspect.isclass(datatype) \
-            and '_wsme_attributes' in datatype.__dict__
+        and '_wsme_attributes' in datatype.__dict__
 
 
 def isarray(datatype):
@@ -193,36 +354,49 @@ def isdict(datatype):
 
 
 def validate_value(datatype, value):
+    if value in (Unset, None):
+        return value
+
+    # Try to promote the data type to one of our complex types.
+    if isinstance(datatype, list):
+        datatype = ArrayType(datatype[0])
+    elif isinstance(datatype, dict):
+        datatype = DictType(*list(datatype.items())[0])
+
+    # If the datatype has its own validator, use that.
     if hasattr(datatype, 'validate'):
         return datatype.validate(value)
-    else:
-        if value in (Unset, None):
-            return value
 
-        if isinstance(datatype, list):
-            datatype = ArrayType(datatype[0])
-        if isinstance(datatype, dict):
-            datatype = DictType(*list(datatype.items())[0])
-        if isarray(datatype):
-            datatype.validate(value)
-        elif isdict(datatype):
-            datatype.validate(value)
-        elif datatype in six.integer_types:
-            if not isinstance(value, six.integer_types):
-                raise ValueError(
-                    "Wrong type. Expected an integer, got '%s'" % (
-                        type(value)
-                    ))
-        elif datatype is text and isinstance(value, bytes):
-            value = value.decode()
-        elif datatype is bytes and isinstance(value, text):
-            value = value.encode()
-        elif not isinstance(value, datatype):
-            raise ValueError(
-                "Wrong type. Expected '%s', got '%s'" % (
-                    datatype, type(value)
-                ))
-        return value
+    # Do type promotion/conversion and data validation for builtin
+    # types.
+    v_type = type(value)
+    if datatype in six.integer_types:
+        if v_type in _promotable_types:
+            try:
+                # Try to turn the value into an int
+                value = datatype(value)
+            except ValueError:
+                # An error is raised at the end of the function
+                # when the types don't match.
+                pass
+    elif datatype is float and v_type in _promotable_types:
+        try:
+            value = float(value)
+        except ValueError:
+            # An error is raised at the end of the function
+            # when the types don't match.
+            pass
+    elif datatype is text and isinstance(value, bytes):
+        value = value.decode()
+    elif datatype is bytes and isinstance(value, text):
+        value = value.encode()
+
+    if not isinstance(value, datatype):
+        raise ValueError(
+            "Wrong type. Expected '%s', got '%s'" % (
+                datatype, v_type
+            ))
+    return value
 
 
 class wsproperty(property):
@@ -230,7 +404,7 @@ class wsproperty(property):
     A specialised :class:`property` to define typed-property on complex types.
     Example::
 
-        class MyComplexType(object):
+        class MyComplexType(wsme.types.Base):
             def get_aint(self):
                 return self._aint
 
@@ -260,20 +434,21 @@ class wsattr(object):
 
     Example::
 
-        class MyComplexType(object):
+        class MyComplexType(wsme.types.Base):
             optionalvalue = int
             mandatoryvalue = wsattr(int, mandatory=True)
             named_value = wsattr(int, name='named.value')
 
-    After inspection, the non-wsattr attributes will be replace, and
+    After inspection, the non-wsattr attributes will be replaced, and
     the above class will be equivalent to::
 
-        class MyComplexType(object):
+        class MyComplexType(wsme.types.Base):
             optionalvalue = wsattr(int)
             mandatoryvalue = wsattr(int, mandatory=True)
 
     """
-    def __init__(self, datatype, mandatory=False, name=None, default=Unset):
+    def __init__(self, datatype, mandatory=False, name=None, default=Unset,
+                 readonly=False):
         #: The attribute name in the parent python class.
         #: Set by :func:`inspect_class`
         self.key = None  # will be set by class inspection
@@ -286,25 +461,38 @@ class wsattr(object):
         #: Default value. The attribute will return this instead
         #: of :data:`Unset` if no value has been set.
         self.default = default
+        #: If True value cannot be set from json/xml input data
+        self.readonly = readonly
 
         self.complextype = None
 
+    def _get_dataholder(self, instance):
+        dataholder = getattr(instance, '_wsme_dataholder', None)
+        if dataholder is None:
+            dataholder = instance._wsme_DataHolderClass()
+            instance._wsme_dataholder = dataholder
+        return dataholder
+
     def __get__(self, instance, owner):
         if instance is None:
             return self
-        return getattr(instance, '_' + self.key, self.default)
+        return getattr(
+            self._get_dataholder(instance),
+            self.key,
+            self.default
+        )
 
     def __set__(self, instance, value):
         try:
             value = validate_value(self.datatype, value)
-        except ValueError:
-            e = sys.exc_info()[1]
-            raise ValueError("%s: %s" % (self.name, e))
+        except ValueError as e:
+            raise exc.InvalidInput(self.name, value, six.text_type(e))
+        dataholder = self._get_dataholder(instance)
         if value is Unset:
-            if hasattr(instance, '_' + self.key):
-                delattr(instance, '_' + self.key)
+            if hasattr(dataholder, self.key):
+                delattr(dataholder, self.key)
         else:
-            setattr(instance, '_' + self.key, value)
+            setattr(dataholder, self.key, value)
 
     def __delete__(self, instance):
         self.__set__(instance, Unset)
@@ -327,7 +515,7 @@ class wsattr(object):
 
     #: attribute data type. Can be either an actual type,
     #: or a type name, in which case the actual type will be
-    #: determined when needed (generaly just before scaning the api).
+    #: determined when needed (generally just before scanning the api).
     datatype = property(_get_datatype, _set_datatype)
 
 
@@ -345,7 +533,7 @@ def sort_attributes(class_, attributes):
     3 mechanisms are attempted :
 
     #.  Look for a _wsme_attr_order attribute on the class_. This allow
-        to define an arbitrary order of the attributes (usefull for
+        to define an arbitrary order of the attributes (useful for
         generated types).
 
     #.  Access the object source code to find the declaration order.
@@ -391,18 +579,17 @@ def inspect_class(class_):
     for name, attr in inspect.getmembers(class_, iswsattr):
         if name.startswith('_'):
             continue
+        if inspect.isroutine(attr):
+            continue
 
-        if isinstance(attr, wsattr):
-            attrdef = attr
-        elif isinstance(attr, wsproperty):
+        if isinstance(attr, (wsattr, wsproperty)):
             attrdef = attr
         else:
             if attr not in native_types and (
                     inspect.isclass(attr)
-                    or isinstance(attr, list)
-                    or isinstance(attr, dict)):
+                    or isinstance(attr, (list, dict))):
                 register_type(attr)
-            attrdef = wsattr(attr)
+            attrdef = getattr(class_, '__wsattrclass__', wsattr)(attr)
 
         attrdef.key = name
         if attrdef.name is None:
@@ -424,12 +611,29 @@ def list_attributes(class_):
     return class_._wsme_attributes
 
 
+def make_dataholder(class_):
+    # the slots are computed outside the class scope to avoid
+    # 'attr' to pullute the class namespace, which leads to weird
+    # things if one of the slots is named 'attr'.
+    slots = [attr.key for attr in class_._wsme_attributes]
+
+    class DataHolder(object):
+        __slots__ = slots
+
+    DataHolder.__name__ = class_.__name__ + 'DataHolder'
+    return DataHolder
+
+
 class Registry(object):
     def __init__(self):
-        self.complex_types = []
+        self._complex_types = []
         self.array_types = set()
         self.dict_types = set()
 
+    @property
+    def complex_types(self):
+        return [t() for t in self._complex_types if t()]
+
     def register(self, class_):
         """
         Make sure a type is registered.
@@ -463,17 +667,53 @@ class Registry(object):
 
         class_._wsme_attributes = None
         class_._wsme_attributes = inspect_class(class_)
+        class_._wsme_DataHolderClass = make_dataholder(class_)
 
         class_.__registry__ = self
-        self.complex_types.append(weakref.ref(class_))
+        self._complex_types.append(weakref.ref(class_))
         return class_
 
+    def reregister(self, class_):
+        """Register a type which may already have been registered.
+        """
+        self._unregister(class_)
+        return self.register(class_)
+
+    def _unregister(self, class_):
+        """Remove a previously registered type.
+        """
+        # Clear the existing attribute reference so it is rebuilt if
+        # the class is registered again later.
+        if hasattr(class_, '_wsme_attributes'):
+            del class_._wsme_attributes
+        # FIXME(dhellmann): This method does not recurse through the
+        # types like register() does. Should it?
+        if isinstance(class_, list):
+            at = ArrayType(class_[0])
+            try:
+                self.array_types.remove(at)
+            except KeyError:
+                pass
+        elif isinstance(class_, dict):
+            key_type, value_type = list(class_.items())[0]
+            self.dict_types = set(
+                dt for dt in self.dict_types
+                if (dt.key_type, dt.value_type) != (key_type, value_type)
+            )
+        # We can't use remove() here because the items in
+        # _complex_types are weakref objects pointing to the classes,
+        # so we can't compare with them directly.
+        self._complex_types = [
+            ct for ct in self._complex_types
+            if ct() is not class_
+        ]
+
     def lookup(self, typename):
         log.debug('Lookup %s' % typename)
         modname = None
         if '.' in typename:
             modname, typename = typename.rsplit('.', 1)
-        for ct in self.complex_types:
+        for ct in self._complex_types:
             ct = ct()
             if ct is not None and typename == ct.__name__ and (
                     modname is None or modname == ct.__module__):
@@ -491,8 +731,9 @@ class Registry(object):
             self.array_types.add(type_)
         elif isinstance(type_, DictType):
             type_ = DictType(
-                    type_.key_type,
-                    self.resolve_type(type_.value_type))
+                type_.key_type,
+                self.resolve_type(type_.value_type)
+            )
             self.dict_types.add(type_)
         else:
             type_ = self.register(type_)
@@ -508,16 +749,17 @@ def register_type(class_):
 
 class BaseMeta(type):
     def __new__(cls, name, bases, dct):
-        if bases[0] is not object and '__registry__' not in dct:
+        if bases and bases[0] is not object and '__registry__' not in dct:
             dct['__registry__'] = registry
         return type.__new__(cls, name, bases, dct)
 
     def __init__(cls, name, bases, dct):
-        if bases[0] is not object:
+        if bases and bases[0] is not object:
             cls.__registry__.register(cls)
 
 
 class Base(six.with_metaclass(BaseMeta)):
+    """Base type for complex types"""
     def __init__(self, **kw):
         for key, value in kw.items():
             if hasattr(self, key):
@@ -549,7 +791,7 @@ class File(Base):
     content = wsproperty(binary, _get_content, _set_content)
 
     def __init__(self, filename=None, file=None, content=None,
-            contenttype=None, fieldstorage=None):
+                 contenttype=None, fieldstorage=None):
         self.filename = filename
         self.contenttype = contenttype
         self._file = file
@@ -559,7 +801,7 @@ class File(Base):
             if fieldstorage.file:
                 self._file = fieldstorage.file
                 self.filename = fieldstorage.filename
-                self.contenttype = unicode(fieldstorage.type)
+                self.contenttype = text(fieldstorage.type)
             else:
                 self._content = fieldstorage.value
 
@@ -568,3 +810,26 @@ class File(Base):
         if self._file is None and self._content:
             self._file = six.BytesIO(self._content)
         return self._file
+
+
+class DynamicBase(Base):
+    """Base type for complex types for which all attributes are not
+    defined when the class is constructed.
+
+    This class is meant to be used as a base for types that have
+    properties added after the main class is created, such as by
+    loading plugins.
+
+    """
+
+    @classmethod
+    def add_attributes(cls, **attrs):
+        """Add more attributes
+
+        The arguments should be valid Python attribute names
+        associated with a type for the new attribute.
+
+        """
+        for n, t in attrs.items():
+            setattr(cls, n, t)
+        cls.__registry__.reregister(cls)
